'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var rxjs = require('rxjs');
var operators = require('rxjs/operators');
var env = require('../../common/env.js');

const defaultSerializer = data => JSON.stringify(data);
const defaultDeserializer = message => message.data;
class IsomorphicWebSocket {
  environment = env.isBrowser() ? 'browser' : 'node';
  websocket;
  open$;
  error$;
  close$;
  message$;
  send$ = new rxjs.Subject();

  get readyState() {
    return this.websocket.readyState;
  }

  constructor({
    url,
    protocols,
    options,
    serializer = defaultSerializer,
    deserializer = defaultDeserializer
  }) {
    this.websocket = this.environment === 'browser' ? new WebSocket(url, protocols) : new (require('ws'))(url, protocols, options);
    this.close$ = rxjs.fromEvent(this.websocket, 'close').pipe(operators.first(), operators.share());
    this.error$ = rxjs.fromEvent(this.websocket, 'error').pipe(operators.first(), operators.shareReplay(), operators.takeUntil(this.close$));
    this.open$ = rxjs.fromEvent(this.websocket, 'open').pipe(operators.first(), operators.shareReplay(), operators.takeUntil(this.close$));
    this.message$ = rxjs.fromEvent(this.websocket, 'message').pipe(operators.map(deserializer), operators.takeUntil(this.close$), operators.shareReplay());
    this.send$.pipe( // TODO(fix)
    // delayWhen(() => this.open$),
    operators.map(serializer), operators.takeUntil(this.close$)).subscribe(message => this.websocket.send(message));
    this.close$.subscribe(() => {
      this.send$.complete();
    });
  }

  get open() {
    return this.open$;
  }

  get error() {
    return this.error$;
  }

  get close() {
    return this.close$;
  }

  get message() {
    return this.message$;
  }

  send(message) {
    this.send$.next(message);
  }

}

exports.IsomorphicWebSocket = IsomorphicWebSocket;
exports.defaultDeserializer = defaultDeserializer;
exports.defaultSerializer = defaultSerializer;
//# sourceMappingURL=isomorphic-websocket.js.map
