import { DefaultHttpClient } from './http-client.js';

const baseURL = 'https://scrapbox.io/api';
const isObject = (data) => typeof data === 'object' && data !== null;
const isNotLoggedInError = (data) => isObject(data) && data['name'] === 'NotLoggedInError';
class RestApi {
    token;
    httpClient;
    constructor(token, httpClient = new DefaultHttpClient()) {
        this.token = token;
        this.httpClient = httpClient;
    }
    async getPage(projectName, pageName) {
        return this.request(`${baseURL}/pages/${encodeURIComponent(projectName)}/${encodeURIComponent(pageName)}?followRename=true`);
    }
    async getProject(projectName) {
        return this.request(`${baseURL}/projects/${encodeURIComponent(projectName)}`);
    }
    async getProjects() {
        return this.request(`${baseURL}/projects`);
    }
    async getMe() {
        return this.request(`${baseURL}/users/me`);
    }
    async request(url) {
        const result = await this.httpClient.get(url, {
            headers: {
                Cookie: `connect.sid=${this.token}`,
            },
        });
        if (isNotLoggedInError(result)) {
            throw new Error('NotLoggedInError');
        }
        return result;
    }
}

export { RestApi, isNotLoggedInError, isObject };
//# sourceMappingURL=rest-api.js.map
