import { Subject, fromEvent } from 'rxjs';
import { first, share, shareReplay, takeUntil, map } from 'rxjs/operators';
import { isBrowser } from '../../common/env.js';

const defaultSerializer = (data) => JSON.stringify(data);
const defaultDeserializer = (message) => message.data;
class IsomorphicWebSocket {
    environment = isBrowser() ? 'browser' : 'node';
    websocket;
    open$;
    error$;
    close$;
    message$;
    send$ = new Subject();
    get readyState() {
        return this.websocket.readyState;
    }
    constructor({ url, protocols, options, serializer = defaultSerializer, deserializer = defaultDeserializer, }) {
        this.websocket = this.environment === 'browser' ? new WebSocket(url, protocols) : new (require('ws'))(url, protocols, options);
        this.close$ = fromEvent(this.websocket, 'close').pipe(first(), share());
        this.error$ = fromEvent(this.websocket, 'error').pipe(first(), shareReplay(), takeUntil(this.close$));
        this.open$ = fromEvent(this.websocket, 'open').pipe(first(), shareReplay(), takeUntil(this.close$));
        this.message$ = fromEvent(this.websocket, 'message').pipe(map(deserializer), takeUntil(this.close$), shareReplay());
        this.send$
            .pipe(
        // TODO(fix)
        // delayWhen(() => this.open$),
        map(serializer), takeUntil(this.close$))
            .subscribe((message) => this.websocket.send(message));
        this.close$.subscribe(() => {
            this.send$.complete();
        });
    }
    get open() {
        return this.open$;
    }
    get error() {
        return this.error$;
    }
    get close() {
        return this.close$;
    }
    get message() {
        return this.message$;
    }
    send(message) {
        this.send$.next(message);
    }
}

export { IsomorphicWebSocket, defaultDeserializer, defaultSerializer };
//# sourceMappingURL=isomorphic-websocket.js.map
