import { packetTypes } from '../constants.js';
import { isIntegerString } from './util.js';

const toSocketIoMessagePayload = (id, data) => {
    return `${packetTypes.send}${id}${JSON.stringify(['socket.io-request', data])}`;
};
const isConnectionMessage = (message) => {
    return message[0] === packetTypes.initialize;
};
const isResponseMessage = (message) => {
    return message[0].length > 2 && message[0].startsWith(packetTypes.response);
};
const getRequestId = (packetType) => packetType.slice(2);
const isResponseMessageOf = (sid) => (message) => {
    return isResponseMessage(message) && sid === getRequestId(message[0]);
};
const isResponseOf = (expected) => ({ sid }) => expected === sid;
// 430[{...}}] => 430, [{}]
// @see https://github.com/socketio/engine.io-protocol
const scrapboxDeserializer = (event) => {
    let packetType = '';
    let message = String(event.data);
    while (message.length) {
        const head = message[0];
        if (isIntegerString(head)) {
            packetType += head;
            message = message.substr(1);
        }
        else {
            return [packetType, JSON.parse(message)];
        }
    }
    return [packetType, undefined];
};
const scrapboxSerializer = (message) => {
    return typeof message === 'string' ? message : JSON.stringify(message);
};

export { getRequestId, isConnectionMessage, isResponseMessage, isResponseMessageOf, isResponseOf, scrapboxDeserializer, scrapboxSerializer, toSocketIoMessagePayload };
//# sourceMappingURL=message.js.map
